;    Various functions: init, (RAM) disk, mmc, timer
;    This file needs to get split up.
;
;    Copyright (C) 2010 Sprite_tm
;    Copyright (C) 2010 Leo C.
;
;    This file is part of avrcpm.
;
;    avrcpm is free software: you can redistribute it and/or modify it
;    under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    avrcpm is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with avrcpm.  If not, see <http://www.gnu.org/licenses/>.
;
;    $Id: remainders.asm 82 2010-08-19 00:02:19Z leo $
;

#define REFR_PRE    8           /* timer prescale factor  1/8 */
#define REFR_CS     0x02        /* timer clock select for 1/8 */
#define REFR_CNT    F_CPU / REFR_RATE / REFR_PRE

	.cseg
regval_tab:
	.db 0,0
	.db 0xFE,0xFC	; _RAS0  _CAS0
	.db 0xF8,0xF4	; _OE    _WE
	.db 255,0	; _255  _0
regval_tab_e:

start:
	ldi temp,low(RAMEND)	; top of memory
	out SPL,temp		; init stack pointer
	ldi temp,high(RAMEND)	; top of memory
	out SPH,temp		; init stack pointer

	clr	_0

; - Kill wdt
	wdr
	out MCUSR,_0

	ldi temp,(1<<WDCE) | (1<<WDE)
	outm8	WDTCSR,temp
	ldi temp,(1<<WDCE)
	outm8	WDTCSR,temp
	
; - Load registers with predefined/initial values

	ldi	zl,low (regval_tab*2)
	ldi	zh,high(regval_tab*2)
	ldi	yl,0
	ldi	yh,0
cp_l:	lpm	xh,z+
	st	y+,xh
	cpi	zl,low(regval_tab_e*2)
	brne	cp_l

; - Clear RAM

	ldi	zl,low(SRAM_START)
	ldi	zh,high(SRAM_START)
	ldi	temp2,high(ramtop)
clr_l:
	st	z+,_0
	cpi	zl,low(ramtop)
	cpc	zh,temp2
	brne	clr_l

; - Setup Ports
	ldi 	temp,(1<<PUD)		;disable pullups
	outm8	P_PUD,temp
	ldi	temp,0xFF
	out 	PORTD,temp		;all pins high
	out 	PORTB,temp
	out 	PORTC,temp
	out 	DDRD,temp		; all outputs
	out	DDRB,temp
	out	DDRC,temp
	cbi	P_RXD-1,RXD		; RXD pin is input

	outm8	TIMSK1,_0
	outm8	TIMSK2,_0
	outm8	TCCR2A,_0
	outm8	TCCR2B,_0


; Init clock/timer system

; Init timer 1 as 1 ms system clock tick.

	ldi	temp2,high(F_CPU/1000)
	ldi	temp, low (F_CPU/1000)
	outm8	OCR1BH,temp2
	outm8	OCR1BL,temp
	ldi	temp,(1<<ICNC1)|(1<<CS10)	;Noise cancel, fall. edge, Normal Mode, clk/1
	outm8	TCCR1B,temp
	inm8	temp,TIMSK1
	ori	temp,(1<<OCIE1B)		;Enable 1ms int.
	outm8	TIMSK1,temp

; - Init serial port

	rcall	uart_init
	
;Init timer2. Refresh-call should happen every (8ms/512)=312 cycles.

	ldi	temp,REFR_CNT*2			; 2 cycles per int
	outm8	OCR2A,temp
	inm8	temp,TCCR2A
	ori	temp,(1<<WGM21)			;CTC mode
	outm8	TCCR2A,temp
	inm8	temp,TCCR2B
	ori	temp,REFR_CS			;clk/REFR_PRE
	outm8	TCCR2B,temp
	inm8	temp,TIMSK2
	ori	temp, (1<<OCIE2A)
	outm8	TIMSK2,temp

	sei


.if BOOTWAIT
	ldi temp,10
	rcall delay_ms

.endif

	rcall printstr
	.db 13,"CPM on an AVR, v1.0",13,0


.if MEMTEST
	rcall printstr
	.db 13,"Testing RAM: fill...",0

;Fill RAM
	ldi xl,0
	ldi xh,0
ramtestw:
	mov temp,xh
	eor temp,xl
	mem_write
	adiw xl,1
	brcc ramtestw
	rcall printstr
	.db "wait...",0

	ldi	temp2,8
ramtestwl:
	ldi	temp,255
	rcall	delay_ms
	dec	temp2
	brne	ramtestwl

	rcall printstr
	.db "reread...",13,0,0

;re-read RAM
	ldi xl,0
	ldi xh,0
ramtestr:
	mem_read
	mov temp2,xh
	eor temp2,xl
	cp temp,temp2
	breq ramtestrok
	rcall printhex
	ldi temp,'<'
	rcall uartPutc
	mov temp,xh
	eor temp,xl
	rcall printhex
	ldi temp,'@'
	rcall uartPutc
	movw temp,x
	rcall printhexw
	ldi temp,13
	rcall uartPutc
ramtestrok:
	adiw xl,1
	brcc ramtestr

.endif

.if MEMFILL_CB
	;Fill ram with cbs, which (for now) will trigger an invalid opcode error.
	ldi xl,0
	ldi xh,0
ramfillw:
	ldi temp,0xcb
	mem_write
	adiw xl,1
	brcc ramfillw
.endif


;----------------------------------------------------------------------------

boot_again:
	rcall printstr
	.db 13,"Initing mmc...",0
	rcall	dsk_partinit
	rcall printstr
	.db 13,"Partinit done.",0

	lds	temp,ndisks
	tst	temp
	brne	boot_ipl2
	rcall	printstr
	.db	"No bootable CP/M disk found! Please change MMC/SD-Card",13,0
	ldi	temp2,18
boot_iplwl:
	ldi	temp,255
	rcall	delay_ms
	dec	temp2
	brne	boot_iplwl
	rjmp	boot_again
	

boot_ipl2:

; Read first sector of first CP/M partition

	lds	xl,hostparttbl
	lds	xh,hostparttbl+1
	lds	yl,hostparttbl+2
	lds	yh,hostparttbl+3
	rcall	mmcReadSect

	rcall	dsk_cboot		;init (de)blocking buffer

;First sector of disk or first CP/M partition is in hostbuf.

;Save to Z80 RAM (only 128 bytes because that's retro)
	ldi zl,low (hostbuf)
	ldi zh,high(hostbuf)
	ldi xl,low (IPLADDR)
	ldi xh,high(IPLADDR)
iplwriteloop:
	ld temp,z+
	mem_write
	adiw xl,1
	cpi zl,low(hostbuf+128)
	brne iplwriteloop
	cpi zh,high(hostbuf+128)
	brne iplwriteloop
	rjmp z80_init


